@@ -6,18 +6,17 @@
 
     Models for the assignment app.
 
-    :copyright: 2011, 2012 by OpenSlides team, see AUTHORS.
+    :copyright: 2011–2013 by OpenSlides team, see AUTHORS.
     :license: GNU GPL, see LICENSE for more details.
 """
 
 from django.core.urlresolvers import reverse
 from django.db import models
-from django.dispatch import receiver
-from django.utils.translation import ugettext_lazy as _, ugettext_noop
+from django.utils.translation import ugettext as _, ugettext_lazy, ugettext_noop
+from django.utils.datastructures import SortedDict
 
 from openslides.utils.person import PersonField
-from openslides.config.models import config
-from openslides.config.signals import default_config_value
+from openslides.config.api import config
 from openslides.projector.api import register_slidemodel
 from openslides.projector.projector import SlideMixin
 from openslides.poll.models import (
@@ -39,19 +38,19 @@ class AssignmentCandidate(models.Model):
 
 
 class Assignment(models.Model, SlideMixin):
-    prefix = 'assignment'
+    prefix = ugettext_noop('assignment')
     STATUS = (
-        ('sea', _('Searching for candidates')),
-        ('vot', _('Voting')),
-        ('fin', _('Finished')),
+        ('sea', ugettext_lazy('Searching for candidates')),
+        ('vot', ugettext_lazy('Voting')),
+        ('fin', ugettext_lazy('Finished')),
     )
 
-    name = models.CharField(max_length=100, verbose_name=_("Name"))
-    description = models.TextField(null=True, blank=True, verbose_name=_("Description"))
-    posts = models.PositiveSmallIntegerField(verbose_name=_("Number of available posts"))
+    name = models.CharField(max_length=100, verbose_name=ugettext_lazy("Name"))
+    description = models.TextField(null=True, blank=True, verbose_name=ugettext_lazy("Description"))
+    posts = models.PositiveSmallIntegerField(verbose_name=ugettext_lazy("Number of available posts"))
     polldescription = models.CharField(
         max_length=100, null=True, blank=True,
-        verbose_name=_("Comment on the ballot paper"))
+        verbose_name=ugettext_lazy("Comment on the ballot paper"))
     status = models.CharField(max_length=3, choices=STATUS, default='sea')
 
     def set_status(self, status):
@@ -175,7 +174,7 @@ class Assignment(models.Model, SlideMixin):
         returns a table represented as a list with all candidates from all
         related polls and their vote results.
         """
-        vote_results_dict = {}
+        vote_results_dict = SortedDict()
         # All polls related to this assigment
         polls = self.poll_set.all()
         if only_published:
@@ -185,6 +184,8 @@ class Assignment(models.Model, SlideMixin):
         for poll in polls:
             options += poll.get_options()
 
+        options.sort(key=lambda option: option.candidate.sort_name)
+
         for option in options:
             candidate = option.candidate
             if candidate in vote_results_dict:
@@ -228,10 +229,10 @@ class Assignment(models.Model, SlideMixin):
         data['template'] = 'projector/Assignment.html'
         return data
 
-    def get_absolute_url(self, link='view'):
-        if link == 'view':
+    def get_absolute_url(self, link='detail'):
+        if link == 'detail' or link == 'view':
             return reverse('assignment_view', args=[str(self.id)])
-        if link == 'edit':
+        if link == 'update' or link == 'edit':
             return reverse('assignment_edit', args=[str(self.id)])
         if link == 'delete':
             return reverse('assignment_delete', args=[str(self.id)])
@@ -241,11 +242,10 @@ class Assignment(models.Model, SlideMixin):
 
     class Meta:
         permissions = (
-            ('can_see_assignment', ugettext_noop("Can see assignment")),
-            ('can_nominate_other',
-                ugettext_noop("Can nominate another person")),
-            ('can_nominate_self', ugettext_noop("Can nominate themselves")),
-            ('can_manage_assignment', ugettext_noop("Can manage assignment")),
+            ('can_see_assignment', ugettext_noop('Can see assignments')),  # TODO: Add plural s to the codestring
+            ('can_nominate_other', ugettext_noop('Can nominate another person')),
+            ('can_nominate_self', ugettext_noop('Can nominate oneself')),
+            ('can_manage_assignment', ugettext_noop('Can manage assignments')),  # TODO: Add plural s also to the codestring
         )
         ordering = ('name',)
 
@@ -308,15 +308,3 @@ class AssignmentPoll(BasePoll, CountInvalid, CountVotesCast, PublishPollMixin):
 
     def __unicode__(self):
         return _("Ballot %d") % self.get_ballot()
-
-
-@receiver(default_config_value, dispatch_uid="assignment_default_config")
-def default_config(sender, key, **kwargs):
-    return {
-        'assignment_publish_winner_results_only': False,
-        'assignment_pdf_ballot_papers_selection': 'CUSTOM_NUMBER',
-        'assignment_pdf_ballot_papers_number': '8',
-        'assignment_pdf_title': _('Elections'),
-        'assignment_pdf_preamble': '',
-        'assignment_poll_vote_values': 'auto',
-    }.get(key)
