@@ -6,24 +6,25 @@
 
     Views for the participant app.
 
-    :copyright: 2011, 2012 by OpenSlides team, see AUTHORS.
+    :copyright: 2011–2013 by OpenSlides team, see AUTHORS.
     :license: GNU GPL, see LICENSE for more details.
 """
 
-# for python 2.5 support
-from __future__ import with_statement
-
-from urllib import urlencode
-
 try:
-    from urlparse import parse_qs
-except ImportError:  # python <= 2.5 grab it from cgi
-    from cgi import parse_qs
+    import qrcode
+except ImportError:
+    draw_qrcode = False
+else:
+    draw_qrcode = True
+
+from cStringIO import StringIO
+from urllib import urlencode
+from urlparse import parse_qs
 
 from reportlab.lib import colors
 from reportlab.lib.units import cm
 from reportlab.platypus import (
-    SimpleDocTemplate, Paragraph, LongTable, Spacer, Table, TableStyle)
+    SimpleDocTemplate, Paragraph, LongTable, Spacer, Table, TableStyle, Image)
 
 from django.contrib import messages
 from django.contrib.auth.decorators import login_required
@@ -31,24 +32,22 @@ from django.contrib.auth.forms import PasswordChangeForm
 from django.contrib.auth.views import login as django_login
 from django.core.urlresolvers import reverse
 from django.shortcuts import redirect
-from django.utils.translation import ugettext as _, ugettext_lazy
+from django.utils.translation import ugettext as _, ugettext_lazy, activate
 
 from openslides.utils.pdf import stylesheet
 from openslides.utils.template import Tab
 from openslides.utils.utils import (
-    template, decodedict, encodedict, delete_default_permissions, html_strong)
+    template, delete_default_permissions, html_strong)
 from openslides.utils.views import (
     FormView, PDFView, CreateView, UpdateView, DeleteView, PermissionMixin,
     RedirectView, SingleObjectMixin, ListView, QuestionMixin, DetailView)
-from openslides.config.models import config
+from openslides.config.api import config
 from openslides.projector.projector import Widget
-from openslides.motion.models import Motion
-from openslides.assignment.models import Assignment
 from openslides.participant.api import gen_username, gen_password, import_users
 from openslides.participant.forms import (
     UserCreateForm, UserUpdateForm, UsersettingsForm,
-    UserImportForm, GroupForm, ConfigForm)
-from openslides.participant.models import User, Group
+    UserImportForm, GroupForm)
+from openslides.participant.models import User, Group, get_protected_perm
 
 
 class UserOverview(ListView):
@@ -60,84 +59,20 @@ class UserOverview(ListView):
     context_object_name = 'users'
 
     def get_queryset(self):
-        try:
-            sortfilter = encodedict(parse_qs(
-                self.request.COOKIES['participant_sortfilter']))
-        except KeyError:
-            sortfilter = {}
-
-        for value in ['gender', 'structure_level', 'type', 'committee', 'status',
-                      'sort', 'reverse']:
-            if value in self.request.REQUEST:
-                if self.request.REQUEST[value] == '---':
-                    try:
-                        del sortfilter[value]
-                    except KeyError:
-                        pass
-                else:
-                    sortfilter[value] = [self.request.REQUEST[value]]
-
         query = User.objects
-        if 'gender' in sortfilter:
-            query = query.filter(gender__iexact=sortfilter['gender'][0])
-        if 'structure_level' in sortfilter:
-            query = query.filter(structure_level__iexact=sortfilter['structure_level'][0])
-        if 'type' in sortfilter:
-            query = query.filter(type__iexact=sortfilter['type'][0])
-        if 'committee' in sortfilter:
-            query = query.filter(committee__iexact=sortfilter['committee'][0])
-        if 'status' in sortfilter:
-            query = query.filter(is_active=sortfilter['status'][0])
-        if 'sort' in sortfilter:
-            if sortfilter['sort'][0] in ['first_name', 'last_name', 'last_login']:
-                query = query.order_by(sortfilter['sort'][0])
-            elif (sortfilter['sort'][0] in
-                    ['structure_level', 'type', 'committee', 'comment']):
-                query = query.order_by(
-                    '%s' % sortfilter['sort'][0])
+        if config['participant_sort_users_by_first_name']:
+            query = query.order_by('first_name')
         else:
-            if config['participant_sort_users_by_first_name']:
-                query = query.order_by('first_name')
-            else:
-                query = query.order_by('last_name')
-
-        if 'reverse' in sortfilter:
-            query = query.reverse()
-
-        self.sortfilter = sortfilter
-
+            query = query.order_by('last_name')
         return query.all()
 
     def get_context_data(self, **kwargs):
         context = super(UserOverview, self).get_context_data(**kwargs)
-
         all_users = User.objects.count()
-
-        # quotient of selected users and all users
-        if all_users > 0:
-            percent = self.object_list.count() * 100 / float(all_users)
-        else:
-            percent = 0
-
-        # list of all existing categories
-        structure_levels = [
-            p['structure_level'] for p in
-            User.objects.values('structure_level').exclude(structure_level='').distinct()]
-        # list of all existing committees
-        committees = [
-            p['committee'] for p in
-            User.objects.values('committee').exclude(committee='').distinct()]
         # context vars
         context.update({
             'allusers': all_users,
-            'request_user': self.request.user,
-            'percent': round(percent, 1),
-            'structure_levels': structure_levels,
-            'committees': committees,
-            'cookie': [
-                'participant_sortfilter', urlencode(decodedict(self.sortfilter),
-                doseq=True)],
-            'sortfilter': self.sortfilter})
+            'request_user': self.request.user})
         return context
 
 
@@ -151,16 +86,6 @@ class UserDetailView(DetailView, PermissionMixin):
     context_object_name = 'shown_user'
 
 
-class GroupDetailView(DetailView, PermissionMixin):
-    """
-    Classed based view to show a specific group in the interface.
-    """
-    permission_required = 'participant.can_manage_participant'
-    model = Group
-    template_name = 'participant/group_detail.html'
-    context_object_name = 'group'
-
-
 class UserCreateView(CreateView):
     """
     Create a new participant.
@@ -170,8 +95,7 @@ class UserCreateView(CreateView):
     model = User
     context_object_name = 'edit_user'
     form_class = UserCreateForm
-    success_url = 'user_overview'
-    apply_url = 'user_edit'
+    success_url_name = 'user_overview'
 
     def manipulate_object(self, form):
         self.object.username = gen_username(
@@ -180,6 +104,17 @@ class UserCreateView(CreateView):
             self.object.default_password = gen_password()
         self.object.set_password(self.object.default_password)
 
+    def post_save(self, form):
+        super(UserCreateView, self).post_save(form)
+        # TODO: find a better solution that makes the following lines obsolete
+        # Background: motion.models.use_post_save adds already the registerd group
+        # to new user but super(..).post_save(form) removes it and sets only the
+        # groups selected in the form (without 'registered')
+        # workaround: add registered group again manually
+        from openslides.participant.api import get_registered_group  # TODO: Test, if global import is possible
+        registered = get_registered_group()
+        self.object.groups.add(registered)
+
 
 class UserUpdateView(UpdateView):
     """
@@ -190,8 +125,26 @@ class UserUpdateView(UpdateView):
     model = User
     context_object_name = 'edit_user'
     form_class = UserUpdateForm
-    success_url = 'user_overview'
-    apply_url = 'participant_edit'
+    success_url_name = 'user_overview'
+
+    def get_form_kwargs(self, *args, **kwargs):
+        form_kwargs = super(UserUpdateView, self).get_form_kwargs(*args, **kwargs)
+        form_kwargs.update({'request': self.request})
+        return form_kwargs
+
+    def manipulate_object(self, form):
+        self.object.username = form.cleaned_data['user_name']
+
+    def post_save(self, form):
+        super(UserUpdateView, self).post_save(form)
+        # TODO: find a better solution that makes the following lines obsolete
+        # Background: motion.models.use_post_save adds already the registerd group
+        # to new user but super(..).post_save(form) removes it and sets only the
+        # groups selected in the form (without 'registered')
+        # workaround: add registered group again manually
+        from openslides.participant.api import get_registered_group  # TODO: Test, if global import is possible
+        registered = get_registered_group()
+        self.object.groups.add(registered)
 
 
 class UserDeleteView(DeleteView):
@@ -200,14 +153,20 @@ class UserDeleteView(DeleteView):
     """
     permission_required = 'participant.can_manage_participant'
     model = User
-    url = 'user_overview'
+    success_url_name = 'user_overview'
 
     def pre_redirect(self, request, *args, **kwargs):
-        if self.get_object() == self.request.user:
+        if self.object == self.request.user:
             messages.error(request, _("You can not delete yourself."))
         else:
             super(UserDeleteView, self).pre_redirect(request, *args, **kwargs)
 
+    def pre_post_redirect(self, request, *args, **kwargs):
+        if self.object == self.request.user:
+            messages.error(self.request, _("You can not delete yourself."))
+        else:
+            super(UserDeleteView, self).pre_post_redirect(request, *args, **kwargs)
+
 
 class SetUserStatusView(RedirectView, SingleObjectMixin):
     """
@@ -215,7 +174,7 @@ class SetUserStatusView(RedirectView, SingleObjectMixin):
     """
     permission_required = 'participant.can_manage_participant'
     allow_ajax = True
-    url = 'user_overview'
+    url_name = 'user_overview'
     model = User
 
     def pre_redirect(self, request, *args, **kwargs):
@@ -224,12 +183,9 @@ class SetUserStatusView(RedirectView, SingleObjectMixin):
         if action == 'activate':
             self.object.is_active = True
         elif action == 'deactivate':
-            if self.get_object().user == self.request.user:
+            if self.object.user == self.request.user:
                 messages.error(request, _("You can not deactivate yourself."))
                 return
-            elif self.get_object().is_superuser:
-                messages.error(request, _("You can not deactivate the administrator."))
-                return
             self.object.is_active = False
         elif action == 'toggle':
             self.object.is_active = not self.object.is_active
@@ -251,8 +207,8 @@ class ParticipantsListPDF(PDFView):
     document_title = ugettext_lazy('List of Participants')
 
     def append_to_pdf(self, story):
-        data = [['#', _('Last Name'), _('First Name'), _('Group'), _('Type'),
-                 _('Committee')]]
+        data = [['#', _('Title'), _('Last Name'), _('First Name'),
+                 _('Structure level'), _('Group'), _('Committee')]]
         if config['participant_sort_users_by_first_name']:
             sort = 'first_name'
         else:
@@ -260,12 +216,17 @@ class ParticipantsListPDF(PDFView):
         counter = 0
         for user in User.objects.all().order_by(sort):
             counter += 1
+            groups = ''
+            for group in user.groups.all():
+                if group.pk != 2:
+                    groups += "%s<br/>" % unicode(_(group.name))
             data.append([
                 counter,
+                Paragraph(user.title, stylesheet['Tablecell']),
                 Paragraph(user.last_name, stylesheet['Tablecell']),
                 Paragraph(user.first_name, stylesheet['Tablecell']),
                 Paragraph(user.structure_level, stylesheet['Tablecell']),
-                Paragraph(user.get_type_display(), stylesheet['Tablecell']),
+                Paragraph(groups, stylesheet['Tablecell']),
                 Paragraph(user.committee, stylesheet['Tablecell'])])
         t = LongTable(data, style=[
             ('VALIGN', (0, 0), (-1, -1), 'TOP'),
@@ -302,13 +263,21 @@ class ParticipantsPasswordsPDF(PDFView):
             sort = 'first_name'
         else:
             sort = 'last_name'
+        # create qrcode image object from system url
+        if draw_qrcode:
+            qrcode_img = qrcode.make(participant_pdf_system_url)
+            img_stream = StringIO()
+            qrcode_img.save(img_stream, 'PNG')
+            img_stream.seek(0)
+            size = 2 * cm
+            I = Image(img_stream, width=size, height=size)
         for user in User.objects.all().order_by(sort):
             cell = []
             cell.append(Spacer(0, 0.8 * cm))
             cell.append(Paragraph(_("Account for OpenSlides"),
-                        stylesheet['Ballot_title']))
+                        stylesheet['Password_title']))
             cell.append(Paragraph(_("for %s") % (user),
-                        stylesheet['Ballot_subtitle']))
+                        stylesheet['Password_subtitle']))
             cell.append(Spacer(0, 0.5 * cm))
             cell.append(Paragraph(_("User: %s") % (user.username),
                         stylesheet['Monotype']))
@@ -316,12 +285,10 @@ class ParticipantsPasswordsPDF(PDFView):
                 Paragraph(
                     _("Password: %s")
                     % (user.default_password), stylesheet['Monotype']))
-            cell.append(Spacer(0, 0.5 * cm))
             cell.append(
-                Paragraph(
-                    _("URL: %s") % (participant_pdf_system_url),
-                    stylesheet['Ballot_option']))
-            cell.append(Spacer(0, 0.5 * cm))
+                Paragraph(participant_pdf_system_url, stylesheet['Monotype']))
+            if draw_qrcode:
+                cell.append(I)
             cell2 = []
             cell2.append(Spacer(0, 0.8 * cm))
             if participant_pdf_welcometext is not None:
@@ -337,6 +304,7 @@ class ParticipantsPasswordsPDF(PDFView):
         # build table
         t = Table(data, 10.5 * cm, 7.42 * cm)
         t.setStyle(TableStyle([
+            ('LEFTPADDING', (0, 0), (0, -1), 30),
             ('LINEBELOW', (0, 0), (-1, 0), 0.25, colors.grey),
             ('LINEBELOW', (0, 1), (-1, 1), 0.25, colors.grey),
             ('LINEBELOW', (0, 1), (-1, -1), 0.25, colors.grey),
@@ -352,6 +320,7 @@ class UserImportView(FormView):
     permission_required = 'participant.can_manage_participant'
     template_name = 'participant/import.html'
     form_class = UserImportForm
+    success_url_name = 'user_overview'
 
     def form_valid(self, form):
         # check for valid encoding (will raise UnicodeDecodeError if not)
@@ -398,6 +367,26 @@ class GroupOverview(ListView):
     model = Group
 
 
+class GroupDetailView(DetailView, PermissionMixin):
+    """
+    Classed based view to show a specific group in the interface.
+    """
+    permission_required = 'participant.can_manage_participant'
+    model = Group
+    template_name = 'participant/group_detail.html'
+    context_object_name = 'group'
+
+    def get_context_data(self, *args, **kwargs):
+        context = super(GroupDetailView, self).get_context_data(*args, **kwargs)
+        query = User.objects
+        if config['participant_sort_users_by_first_name']:
+            query = query.order_by('first_name')
+        else:
+            query = query.order_by('last_name')
+        context['group_members'] = query.filter(django_user__groups__in=[context['group']])
+        return context
+
+
 class GroupCreateView(CreateView):
     """
     Create a new group.
@@ -407,8 +396,7 @@ class GroupCreateView(CreateView):
     context_object_name = 'group'
     model = Group
     form_class = GroupForm
-    success_url = 'user_group_overview'
-    apply_url = 'user_group_edit'
+    success_url_name = 'user_group_overview'
 
     def get(self, request, *args, **kwargs):
         delete_default_permissions()
@@ -424,54 +412,52 @@ class GroupUpdateView(UpdateView):
     model = Group
     context_object_name = 'group'
     form_class = GroupForm
-    success_url = 'user_group_overview'
-    apply_url = 'user_group_edit'
+    success_url_name = 'user_group_overview'
 
     def get(self, request, *args, **kwargs):
         delete_default_permissions()
         return super(GroupUpdateView, self).get(request, *args, **kwargs)
 
+    def get_form_kwargs(self, *args, **kwargs):
+        form_kwargs = super(GroupUpdateView, self).get_form_kwargs(*args, **kwargs)
+        form_kwargs.update({'request': self.request})
+        return form_kwargs
+
 
 class GroupDeleteView(DeleteView):
     """
-    Delete a Group.
+    Delete a group.
     """
     permission_required = 'participant.can_manage_participant'
     model = Group
-    url = 'user_group_overview'
+    success_url_name = 'user_group_overview'
 
     def pre_redirect(self, request, *args, **kwargs):
-        if self.get_object().name.lower() in ['anonymous', 'registered']:
-            messages.error(request, _("You can not delete this Group."))
-        else:
+        if not self.is_protected_from_deleting():
             super(GroupDeleteView, self).pre_redirect(request, *args, **kwargs)
 
+    def pre_post_redirect(self, request, *args, **kwargs):
+        if not self.is_protected_from_deleting():
+            super(GroupDeleteView, self).pre_post_redirect(request, *args, **kwargs)
 
-class Config(FormView):
-    """
-    Config page for the participant app.
-    """
-    permission_required = 'config.can_manage_config'
-    form_class = ConfigForm
-    template_name = 'participant/config.html'
-
-    def get_initial(self):
-        return {
-            'participant_pdf_system_url': config['participant_pdf_system_url'],
-            'participant_pdf_welcometext': config['participant_pdf_welcometext'],
-            'participant_sort_users_by_first_name': config['participant_sort_users_by_first_name']}
-
-    def form_valid(self, form):
-        config['participant_pdf_system_url'] = (
-            form.cleaned_data['participant_pdf_system_url'])
-        config['participant_pdf_welcometext'] = (
-            form.cleaned_data['participant_pdf_welcometext'])
-        config['participant_sort_users_by_first_name'] = (
-            form.cleaned_data['participant_sort_users_by_first_name'])
-        messages.success(
-            self.request,
-            _('Participants settings successfully saved.'))
-        return super(Config, self).form_valid(form)
+    def is_protected_from_deleting(self):
+        """
+        Checks whether the group is protected.
+        """
+        if self.object.pk in [1, 2]:
+            messages.error(request, _('You can not delete this group.'))
+            return True
+        if (not self.request.user.is_superuser and
+            get_protected_perm() in self.object.permissions.all() and
+            not Group.objects.exclude(pk=self.object.pk).filter(
+                permissions__in=[get_protected_perm()],
+                user__pk=self.request.user.pk).exists()):
+            messages.error(
+                self.request,
+                _('You can not delete the last group containing the permission '
+                  'to manage participants you are in.'))
+            return True
+        return False
 
 
 def login(request):
@@ -502,15 +488,21 @@ def user_settings(request):
     if request.method == 'POST':
         form_user = UsersettingsForm(request.POST, instance=request.user)
         if form_user.is_valid():
-            form_user.save()
+            user = form_user.save(commit=False)
+            user.username = form_user.cleaned_data['user_name']
+            user.save()
+            language = request.LANGUAGE_CODE = \
+                request.session['django_language'] = form_user.cleaned_data['language']
+            activate(language)
             messages.success(request, _('User settings successfully saved.'))
         else:
             messages.error(request, _('Please check the form for errors.'))
     else:
-        form_user = UsersettingsForm(instance=request.user)
+        language = request.session.get('django_language', request.LANGUAGE_CODE)
+        form_user = UsersettingsForm(instance=request.user, initial={'language': language})
 
     return {
-        'form_user': form_user,
+        'form': form_user,
         'edituser': request.user,
     }
 
@@ -526,7 +518,7 @@ def user_settings_password(request):
         if form.is_valid():
             form.save()
             messages.success(request, _('Password successfully changed.'))
-            return redirect(reverse('user_settings'))
+            return redirect(reverse('dashboard'))
         else:
             messages.error(request, _('Please check the form for errors.'))
     else:
@@ -539,11 +531,12 @@ def user_settings_password(request):
 
 def register_tab(request):
     """
-    Register the participant tab.
+    Registers the participant tab.
     """
     selected = request.path.startswith('/participant/')
     return Tab(
         title=_('Participants'),
+        app='participant',
         url=reverse('user_overview'),
         permission=(
             request.user.has_perm('participant.can_see_participant') or
@@ -553,34 +546,10 @@ def register_tab(request):
 
 def get_widgets(request):
     """
-    Returns all widgets of the participant app. This is a user_widget, a
-    group_widget and a personal_info_widget.
+    Returns all widgets of the participant app. This is a user_widget
+    and a group_widget.
     """
-    return [
-        get_personal_info_widget(request),
-        get_user_widget(request),
-        get_group_widget(request)]
-
-
-def get_personal_info_widget(request):
-    """
-    Provides a widget for personal info. It shows your submitted motions
-    and where you are supporter or candidate.
-    """
-    personal_info_context = {
-        'submitted_motions': Motion.objects.filter(submitter=request.user),
-        'config_motion_min_supporters': config['motion_min_supporters'],
-        'supported_motions': Motion.objects.filter(motionsupporter=request.user),
-        'assignments': Assignment.objects.filter(
-            assignmentcandidate__person=request.user,
-            assignmentcandidate__blocked=False)}
-    return Widget(
-        name='personal_info',
-        display_name=_('My motions and elections'),
-        template='participant/personal_info_widget.html',
-        context=personal_info_context,
-        permission_required=None,
-        default_column=1)
+    return [get_user_widget(request), get_group_widget(request)]
 
 
 def get_user_widget(request):
@@ -589,6 +558,7 @@ def get_user_widget(request):
     user slides.
     """
     return Widget(
+        request,
         name='user',
         display_name=_('Participants'),
         template='participant/user_widget.html',
@@ -603,6 +573,7 @@ def get_group_widget(request):
     group slides.
     """
     return Widget(
+        request,
         name='group',
         display_name=_('Groups'),
         template='participant/group_widget.html',
